<?php

namespace Tvip\CoreBundle\Controller\ProviderApi;

use Doctrine\Common\Util\Inflector;
use Doctrine\ORM\ORMException;
use FOS\RestBundle\Request\ParamFetcher;
use FOS\RestBundle\Util\Codes;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;
use Tvip\CoreBundle\Entity\Account;
use FOS\RestBundle\Controller\Annotations as Rest;
use Tvip\CoreBundle\Entity\AccountSubscription;
use Tvip\CoreBundle\Entity\Device;
use Tvip\CoreBundle\Entity\MessageCommand;
use Tvip\CoreBundle\Entity\Tarif;
use Tvip\CoreBundle\Util\UniqueIdConverter;

class AccountController extends BaseRestController
{

    /**
     * Get the list of accounts

     * @FOS\RestBundle\Controller\Annotations\QueryParam(name="provider", requirements="\d+", description="Filter by provider")
     * @FOS\RestBundle\Controller\Annotations\QueryParam(name="remote_custom_field", description="Filter by remote custom field")
     * @FOS\RestBundle\Controller\Annotations\QueryParam(name="remote_id", requirements="\d+", description="Filter by remote id")
     * @FOS\RestBundle\Controller\Annotations\QueryParam(name="start", requirements="\d+", default="0", description="Start item")
     * @FOS\RestBundle\Controller\Annotations\QueryParam(name="limit", requirements="\d+", default="10", description="Number of items")
     * @FOS\RestBundle\Controller\Annotations\QueryParam(name="sort", array=true, requirements="ASC|DESC", nullable=true, strict=true, description="Order by array (key is field, value is direction)")
     *
     * @param ParamFetcher $paramFetcher
     *
     * @return mixed
     */
    public function cgetAction(ParamFetcher $paramFetcher){
        return parent::cgetAction($paramFetcher);
    }
    
    /**
     * Get single account
     * @param $id
     * @return \FOS\RestBundle\View\View
     */
    public function getAction($id){
        return parent::getAction($id);
    }

    /**
     * @param $id
     * @param Request $request
     * @return \FOS\RestBundle\View\View
     */
    public function putAction($id, Request $request){
        return parent::putAction($id,$request);
    }

    /**
     * Delete account
     * @param $id
     * @return \FOS\RestBundle\View\View
     */
    public function deleteAction($id){
        return parent::deleteAction($id);
    }

    /**
     * @param $id
     * @param Request $request
     * @return \FOS\RestBundle\View\View
     * @throws ProviderApiIncorrectSyntaxException
     */
    public function postMessageAction($id, Request $request){
        $content = $request->getContent();
        $json = json_decode($content);
        if(!$json) throw new ProviderApiIncorrectSyntaxException('Message must have valid JSON in POST request');
        $message = $this->parseJsonMessage($json);
        $messages = $this->sendAccountMessage($this->getEntityById($id),$message);
        return $this->view(array('data'=>$messages));

    }

    /**
     * @param $id
     * @param Request $request
     * @return \FOS\RestBundle\View\View
     * @throws ProviderApiIncorrectSyntaxException
     */
    public function putPinAction($id,Request $request){
        $content = $request->getContent();
        $json = json_decode($content);
        if(!$json) throw new ProviderApiIncorrectSyntaxException('Must have valid JSON in PUT request');

        if(!property_exists($json,'pin')) throw new ProviderApiIncorrectSyntaxException('Body must have pin property');
        /**
         * @var $account Account
         */
        $account = $this->getEntityById($id);

        $account->setPin($json->pin);
        $this->getEntityManager()->flush();
        return $this->redirectToGetView($account,Codes::HTTP_ACCEPTED);

    }

    /**
     * @param $id
     * @param Request $request
     * @return \FOS\RestBundle\View\View
     * @throws ProviderApiIncorrectSyntaxException
     */
    public function putPinMd5Action($id,Request $request){
        $content = $request->getContent();
        $json = json_decode($content);
        if(!$json) throw new ProviderApiIncorrectSyntaxException('Must have valid JSON in PUT request');

        if(!property_exists($json,'pin')) throw new ProviderApiIncorrectSyntaxException('Body must have pin property');
        /**
         * @var $account Account
         */
        $account = $this->getEntityById($id);

        $account->setPinMd5($json->pin);
        $this->getEntityManager()->flush();
        return $this->redirectToGetView($account,Codes::HTTP_ACCEPTED);

    }


    /**
     * @param $json
     * @return MessageCommand
     * @throws ProviderApiIncorrectSyntaxException
     */
    private  function parseJsonMessage($json){
        if(!property_exists($json,'type')) throw  new ProviderApiIncorrectSyntaxException('Message must have type field');
        if(!property_exists($json,'text')) throw  new ProviderApiIncorrectSyntaxException('Message must have text field');
        if(!in_array($json->type,array('notify','confirm'))) throw  new ProviderApiIncorrectSyntaxException('Message must have correct type');
        if(property_exists($json,'delay')) $json->delay = 10;

        $message = new MessageCommand();
        $message->setType($json->type);
        if(property_exists($json,'title')) $message->setTitle($json->title);
        $message->setText($json->text);
        if(property_exists($json,'delay')) $message->setDelay($json->delay);

        return $message;

    }
    
    /**
     * @param Account $account
     * @param MessageCommand $messageCommand
     * @return MessageCommand[]
     */
    private function sendAccountMessage(Account $account, MessageCommand $messageCommand){

        $messages  = array();
        if($account->getDeviceCount()>0){
            foreach ($account->getDevices() as $device) {
                $message = clone($messageCommand);
                $message->setDevice($device);
                $this->getEntityManager()->persist($message);
                array_push($messages,$message);
            }
            $this->getEntityManager()->flush();
        }

        return $messages;

    }

    /**
     * @param $id
     * @return \FOS\RestBundle\View\View
     * Enable account via create new Account Status

     */
    public function putEnableAction($id){
        /**
         * @var $entity Account
         */
        $entity = $this->getEntityById($id);
        $entity->setEnabled(true);
        $this->getEntityManager()->flush($entity);
        return $this->redirectToGetView($entity,Codes::HTTP_ACCEPTED);


    }

    /**
     * @param $id
     * @return \FOS\RestBundle\View\View
     *
     * Disable account via create new Account Status
     */
    public function putDisableAction($id){
        /**
         * @var $entity Account
         */
        $entity = $this->getEntityById($id);
        $entity->setEnabled(false);
        $this->getEntityManager()->flush($entity);
        return $this->redirectToGetView($entity,Codes::HTTP_ACCEPTED);

    }

    /**
     * @param $id
     * @return \FOS\RestBundle\View\View
     */
    public function getDevicesAction($id){
        /**
         * @var $entity Account
         */
        $entity = $this->getEntityById($id);
        $devices = $this->getEntityManager()->getRepository('TvipCoreBundle:Device')->findBy(array('account'=>$entity));
        return $this->view(array('data'=>$devices));
    }

    /**
     * @param $id
     * @return \FOS\RestBundle\View\View

     */
    public function getSubscriptionsAction($id){
        /**
         * @var $entity Account
         */
        $entity = $this->getEntityById($id);
        return $this->view(array('data'=>$entity->getCurrentAccountSubscriptions()));
    }
    
    /**
     * @param $id
     * @param Request $request
     * @return \FOS\RestBundle\View\View
     * @throws ProviderApiConflictException
     * @throws ProviderApiIncorrectSyntaxException
     *
     * Bind existed or create new Device to Account and return Location to new Device if success, on error return HTTP error
     * Request MUST contain mac_addr or ID of device
     */
    public function putBindDeviceAction($id,Request $request){

        /**
         * @var $account Account
         */
        $account = $this->getEntityById($id);

        $device = $this->lookupDevice($request);

        if(!$device){
            $serialized = json_decode($request->getContent());
            if($serialized->unique_id){
                $device = new Device();
                $device->setUniqueId(UniqueIdConverter::convert($serialized->unique_id));
            }else{
                throw  new ProviderApiIncorrectSyntaxException('id or mac-address must be present in request');
            }
        }
        if($device->getProvider() && $device->getProvider()->getId() !== $account->getProvider()->getId()) throw new ProviderApiConflictException('Device provider not equal with Account provider');
        if($device->getAccount()) throw new ProviderApiConflictException('Device already binded to account '.$device->getAccount()->getId());

        if(!$account->isAllowToBindDevice()) throw  new ProviderApiConflictException('Account device count exceeded');
        $device->setAccount($account);
        $this->getEntityManager()->persist($device);
        $this->getEntityManager()->persist($account);
        $this->getEntityManager()->flush();
        return $this->redirectToGetView($device,Codes::HTTP_ACCEPTED);
    }


    /**
     * @param $id
     * @param Request $request
     * @return \FOS\RestBundle\View\View
     * @throws ProviderApiConflictException
     * @throws ProviderApiIncorrectSyntaxException

     * Unbind existed Device from Account and return Location to Device if success, on error return HTTP error
     * Request MUST contain mac_addr or|and ID of device
     */
    public function putUnbindDeviceAction($id,Request $request){
        $account = $this->getEntityById($id);
        $device = $this->lookupDevice($request);
        if(!$device) return $this->view('Device not found',Codes::HTTP_NOT_FOUND);
        if($device->getAccount()==null) throw new ProviderApiConflictException('Device already unbinded');
        if($device->getAccount() && $device->getAccount()->getId()==$id){
            $device->setAccount(null);
            $this->getEntityManager()->flush();
            return $this->redirectToGetView($account,Codes::HTTP_ACCEPTED);
        }else{
            throw new ProviderApiConflictException('Device binded to another account with id '.$device->getAccount()->getId());
        }
    }

    /**
     * @param $id
     * @param Request $request
     * @return \FOS\RestBundle\View\View
     * @throws ProviderApiConflictException
     * @throws ProviderApiIncorrectSyntaxException Add tarif to account
     */
    public function putBindTarifAction($id, Request $request){
        /**
         * @var $account Account
         */
        $account = $this->getEntityById($id);
        $tarif = $this->lookupTarif($request);
        if(!$tarif) throw new NotFoundHttpException('Tarif not found');
        if( $tarif->getProvider()->getId() !== $account->getProvider()->getId()) throw new ProviderApiConflictException('Provider of tarif is not euqal Provider of Account');
        foreach($account->getCurrentAccountSubscriptions() as $accountSubscription){
            if(!$accountSubscription->getStop() && $accountSubscription->getTarif() && $accountSubscription->getTarif()->getId() == $tarif->getId()) throw new ProviderApiIncorrectSyntaxException('Tarif already binded');
        }

        $accountSubscription = new AccountSubscription();
        $accountSubscription->setAccount($account);
        $accountSubscription->setStart(new \DateTime());
        $accountSubscription->setTarif($tarif);
        $this->getEntityManager()->persist($accountSubscription);
        $this->getEntityManager()->flush();

        return $this->redirectToGetView($account,Codes::HTTP_ACCEPTED);

    }

    /**
     * @param $id
     * @param Request $request
     * @return \FOS\RestBundle\View\View
     * @throws ProviderApiIncorrectSyntaxException
     * Cancel tarif on Accound
     * Request MUST contain id of Tarif
     */
    public function putUnbindTarifAction($id, Request $request){
        /**
         * @var $account Account
         */
        $account = $this->getEntityById($id);
        $tarif = $this->lookupTarif($request);
        if(!$tarif) throw new NotFoundHttpException('Tarif not found');
        foreach($account->getCurrentAccountSubscriptions() as $accountSubscription){
            if(
                $accountSubscription->getTarif() &&
                $accountSubscription->getTarif()->getId() == $tarif->getId() &&
                !$accountSubscription->getStop()
            ){
                $accountSubscription->setStop(new \DateTime());
                $this->getEntityManager()->persist($accountSubscription);
                $this->getEntityManager()->flush();
                return $this->redirectToGetView($account,Codes::HTTP_ACCEPTED);
            }
        }
        $this->getEntityManager()->flush();

        return $this->redirectToGetView($account,Codes::HTTP_NOT_FOUND);
    }


    /**
     * @param Request $request
     * @return bool|Device
     * @throws ProviderApiIncorrectSyntaxException
     * @throw ORMException
     */
    private function lookupDevice(Request $request){
        $serialized = json_decode($request->getContent());
        foreach ($serialized as $name=>$value){
            if($name=='unique_id') $value = UniqueIdConverter::convert($value);
            $serialized->{$name}=$value;
        }
        if(!$serialized) throw new ProviderApiIncorrectSyntaxException('');
        $criteria = array();
        foreach($serialized as $name=>$value){
            $criteria[Inflector::camelize($name)]=$value;
        }
        try{
            $device = $this->getEntityManager()->getRepository('TvipCoreBundle:Device')->findOneBy($criteria);
        }catch (ORMException $e){
            throw new ProviderApiIncorrectSyntaxException($e->getMessage());
        }

        return $device!=null ? $device: false;

    }

    /**
     * @param Request $request
     * @return bool|Tarif
     * @throws ProviderApiIncorrectSyntaxException
     */
    private function lookupTarif(Request $request){
        $serialized = json_decode($request->getContent());
        if(!$serialized) throw new ProviderApiIncorrectSyntaxException('');
        $criteria = array();
        foreach($serialized as $name=>$value){
            $criteria[Inflector::camelize($name)]=$value;
        }
        try{
            $tarif = $this->getEntityManager()->getRepository('TvipCoreBundle:Tarif')->findOneBy($criteria);
        }catch (ORMException $e){
            throw new ProviderApiIncorrectSyntaxException($e->getMessage());
        }

        return $tarif !== null ? $tarif: false;
    }
}